
/* Chrysalide - Outil d'analyse de fichiers binaires
 * rename.c - prototypes pour l'équivalent Python du fichier "arch/operands/rename.c"
 *
 * Copyright (C) 2020 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "rename.h"


#include <pygobject.h>


#include <arch/operands/rename.h>


#include "../../access.h"
#include "../../helpers.h"



/* ------------------------ INTERFACE POUR OPERANDE RENOMMEE ------------------------ */


#define RENAMED_OPERAND_DOC                                             \
    "The RenamedOperand interface depicts operands renamed with an"     \
    " alternative text."


/* Fournit un texte comme représentation alternative d'opérande. */
static PyObject *py_renamed_operand_get_text(PyObject *, void *);



/* ----------------------- INTERFACE POUR OPERANDE RENOMMABLE ----------------------- */


#define RENAMEABLE_OPERAND_DOC                                          \
    "The RenameableOperand interface depicts operands which can get"    \
    " renamed with an alternative text."                                \


/* Construit un opérande de représentation alternative. */
static PyObject *py_renameable_operand_build(PyObject *, PyObject *);



/* ---------------------------------------------------------------------------------- */
/*                          INTERFACE POUR OPERANDE RENOMMEE                          */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit un texte comme représentation alternative d'opérande.*
*                                                                             *
*  Retour      : Chaîne de caractère de représentation alternative.           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_renamed_operand_get_text(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GRenamedOperand *operand;               /* Elément à consulter         */
    const char *text;                       /* Texte alternatif de rendu   */

#define RENAMED_OPERAND_TEXT_ATTRIB PYTHON_GET_DEF_FULL     \
(                                                           \
    text, py_renamed_operand,                               \
    "Alternative text for the operand rendering."           \
)

    operand = G_RENAMED_OPERAND(pygobject_get(self));
    text = g_renamed_operand_get_text(operand);

    result = PyUnicode_FromString(text);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_renamed_operand_type(void)
{
    static PyMethodDef py_renamed_operand_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_renamed_operand_getseters[] = {
        RENAMED_OPERAND_TEXT_ATTRIB,
        { NULL }
    };

    static PyTypeObject py_renamed_operand_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.arch.operands.RenamedOperand",
        .tp_basicsize   = sizeof(PyObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = RENAMED_OPERAND_DOC,

        .tp_methods     = py_renamed_operand_methods,
        .tp_getset      = py_renamed_operand_getseters

    };

    return &py_renamed_operand_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....RenamedOperand'.   *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_renamed_operand_is_registered(void)
{
    PyTypeObject *type;                     /* Type 'RenamedOperand'       */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_renamed_operand_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.arch.operands");

        dict = PyModule_GetDict(module);

        if (!register_interface_for_pygobject(dict, G_TYPE_RENAMED_OPERAND, type))
            return false;

    }

    return true;

}



/* ---------------------------------------------------------------------------------- */
/*                         INTERFACE POUR OPERANDE RENOMMABLE                         */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : self = contenu binaire à manipuler.                          *
*                args = argument accompagnant l'appel.                        *
*                                                                             *
*  Description : Construit un opérande de représentation alternative.         *
*                                                                             *
*  Retour      : Nouvel opérande, en version renommée.                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_renameable_operand_build(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    const char *text;                       /* Texte alternatif à utiliser */
    int ret;                                /* Bilan de lecture des args.  */
    GRenameableOperand *operand;            /* Instance à manipuler        */
    GRenamedOperand *renamed;               /* Instance nouvelle           */

#define RENAMEABLE_OPERAND_BUILD_METHOD PYTHON_METHOD_DEF           \
(                                                                   \
    build, "$self, text",                                           \
    METH_VARARGS, py_renameable_operand,                            \
    "Build a new operand with an alternative text as rendering."    \
    "\n"                                                            \
    "The result of the call is an object implementing the"          \
    " pychrysalide.arch.operands.RenamedOperand interface."         \
)

    ret = PyArg_ParseTuple(args, "s", &text);
    if (!ret) return NULL;

    operand = G_RENAMEABLE_OPERAND(pygobject_get(self));

    renamed = g_renameable_operand_build(operand, text);

    result = pygobject_new(G_OBJECT(renamed));
    g_object_unref(G_OBJECT(renamed));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_renameable_operand_type(void)
{
    static PyMethodDef py_renameable_operand_methods[] = {
        RENAMEABLE_OPERAND_BUILD_METHOD,
        { NULL }
    };

    static PyGetSetDef py_renameable_operand_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_renameable_operand_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.arch.operands.RenameableOperand",
        .tp_basicsize   = sizeof(PyObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = RENAMEABLE_OPERAND_DOC,

        .tp_methods     = py_renameable_operand_methods,
        .tp_getset      = py_renameable_operand_getseters

    };

    return &py_renameable_operand_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....RenameableOperand'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_renameable_operand_is_registered(void)
{
    PyTypeObject *type;                     /* Type 'RenameableOperand'    */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_renameable_operand_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.arch.operands");

        dict = PyModule_GetDict(module);

        if (!register_interface_for_pygobject(dict, G_TYPE_RENAMEABLE_OPERAND, type))
            return false;

    }

    return true;

}
